#
# db2_install::software
#
# @summary This class install's specified version of the the DB2 software on the system.
# The source zip file from IBM is found at the location specified at `source`. By default it will install only the minimal required set. 
# 
# Here is an example on how to use it:
# 
# ``` puppet
# class { '::db2_install::software':
#   source     => '/software',
#   version    => '11.5',
#   location   => '/opt/ibm/db2/V11.5',
#   file_name  => 'DB2S_11.5.4_MPML.tar.gz',
#   version    => '9.0.0.0'
#   temp_dir   => '/my_tmp'
#   logoutput  => 'true,
# }
# ```
# 
# On systems with a secured `/tmp` direcory, you MUST specify the `temp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted DB2 installation kit.
#
# @param [Enum['present', 'absent']] ensure
#    State to obtain.
#    The ensure attribute can be one of two values:
#    - present
#    - absent
#    When you specify `present`, Puppet will make sure the resource is available with all specified options and properties.
#    When the resource is already available(installed), and all attributes are as the are specified, Puppet will do nothing.
#    When you specify `absent`, Puppet will remove the resource if it is available. If it is not installed, Puppet will do nothing.
#
# @param [Db2_install::Version] version
#    The version to be installed.
#    Here is an example on how to use this:
#        class { '::db2_install::software':
#          ...
#          version => '11.0.0.0',
#          ...
#        }
#
# @param [String[1]] source
#    The location of the DB2 software.
#    Here is an example on how to use this:
#        class { '::db2_install::...':
#          ...
#          source => '/software',
#          ...
#        }
#
# @param [Stdlib::Absolutepath] location
#    The location (e.
#    g. folder) that you want the software to be installed in.
#
# @param [Stdlib::Absolutepath] temp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::db2_install::...':
#          ...
#          temp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [String[1]] file_name
#    The file containing the install kit for the DB2 software.
#
# @param [Array[String[1]]] components
#    The DB2 component you want to install.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::db2_install::...':
#          ...
#          logoutput => true,
#          ...
#        }
#
# @param [Boolean] allow_insecure
#    When set to `true` Ignore HTTPS certificate errors
#    (default: false)
#
#
# See the file "LICENSE" for the full license governing this code.
#
define db2_install::software (
  Enum['present','absent']             $ensure         = 'present',
  Db2_install::Version                 $version        = undef,
  String[1]                            $source         = undef,
  Stdlib::Absolutepath                 $location       = undef,
  Stdlib::Absolutepath                 $temp_dir       = '/tmp',
  String[1]                            $file_name      = undef,
  Array[String[1]]                     $components     = lookup('db2_install::components'),
  Variant[Boolean, Enum['on_failure']] $logoutput      = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  Boolean                              $allow_insecure = lookup({ name => 'allow_insecure', default_value => false }),
) {
  db2_install { "software::${title}": ensure => 'present' }

  $container_name = case $version {
    '10.5': { 'server' }
    '11.5': { 'server_dec' }
    '11.5.6', '11.5.7': { 'universal' }
    default: { fail "${version} is an unsupoorted version number." }
  }
  $version_parts = $version.split('\.')
  $major_version = $version_parts[0]
  $minor_version = $version_parts[1]
  $short_version = "${major_version}.${minor_version}"

  $sanitized_name = regsubst($title, / |\/|-|\./, '_', 'G')

  easy_type::debug_evaluation() # Show local variable on extended debug

  $components_to_add    = $components - db2_install::installed_components($location)
  $components_to_remove = db2_install::installed_components($location) - $components

  if $short_version != $facts.dig('db2_install_locations','product_version', $location, 'short_version') {
    if $ensure == 'present' {
      file { "${temp_dir}/${sanitized_name}_extracted":
        ensure => 'directory',
      }

      archive { "${temp_dir}/${sanitized_name}.tar.gz":
        ensure         => present,
        allow_insecure => $allow_insecure,
        cleanup        => true,
        extract        => true,
        extract_path   => "${temp_dir}/${sanitized_name}_extracted",
        source         => "${source}/${file_name}",
        temp_dir       => $temp_dir,
      }

      file { "${temp_dir}/${sanitized_name}.rsp":
        ensure       => 'file',
        content      => epp("db2_install/${major_version}/db2server.rsp.epp",
          location   => $location,
          components => $components,
        ),
      }
      -> exec { "Install ${title}":
        command   => "${temp_dir}/${sanitized_name}_extracted/${container_name}/db2setup -f sysreq -r ${temp_dir}/${sanitized_name}.rsp",
        logoutput => $logoutput,
        require   => Archive["${temp_dir}/${sanitized_name}.tar.gz"],
      }

      cleanup { "Temp files for ${title}":
        file_name => [
          "${temp_dir}/${sanitized_name}",
          "${temp_dir}/${sanitized_name}_extracted",
          "${temp_dir}/${file_name}",
          "${temp_dir}/${sanitized_name}.rsp",
        ],
      }
    } else {
      warning 'Absenting not (yet) implemented'
    }
  } else {
    # Correct version already installed
    if $components_to_add != [] {
      # TODO: Add components installation
      debug("Adding components ${components_to_add.join(',')} not (yet) implemented.")
    }
    if $components_to_remove != [] {
      # TODO: remove components installation
      debug("Removing components ${$components_to_remove.join(',')}not (yet) implemented.")
    }
  }
}
